@@ -5,6 +5,7 @@
  *        3   simpson
  *        4   ISAR
  *        5   MCI
+ *        6   biomass sum
  *
  * Supports Geometric and k-nn graphs, and toroidal correction
  * TODO: border correction
@@ -22,6 +23,7 @@
 #include "isar.h"
 #include "mci.h"
 #include "mean_sd.h"
+#include "biomass.h"
 #ifndef FUN_H_
 #define FUN_H_
 
@@ -31,9 +33,11 @@ class Fun
 	Graph *graph;
 	std::vector<std::vector <double> > value;
 	std::vector<double> parvec;
+
 	int *gtype; // 0 = geometric, 1 = knn
 	int *ftype;
 	int *included;
+	int *trans; // translation weights should be computed
 	double *fpar;
 
 	int *dbg;
@@ -41,10 +45,10 @@ class Fun
 public:
 	Fun();
 	virtual ~Fun();
-	void Init(Graph *g0, double *par0, int *parn, int *gt, int *ft, double *fpar, int *included0, int *dbg0);
+	void Init(Graph *g0, double *par0, int *parn, int *gt, int *ft, double *fpar, int *trans0, int *included0, int *dbg0);
 	void calculate();
 	void re_calculate();
-	SEXP toSEXP();
+	SEXP toSEXP(SEXP);
 };
 
 #endif /*FUN_H_*/
